﻿/*
 * This method of enumerables makes it so that GetCustomers() is run twice, 
 * doing the same read line and return of the same data, for two different 
 * proccesses. Since customers is not immediately used, the function isn't 
 * happening yet. Count() causes the first run of GetCustomers() in order
 * to get the customers to receive the count. GetCustomers() is then rerun
 * during the Console.WriteLine() where it retrieves each customer.
 */

Console.WriteLine("_______________________________________");
Console.WriteLine("STANDARD");
Console.WriteLine();


var customers = GetCustomers();                     


var count = customers.Count();
Console.WriteLine($"\tThere are {count} customers");

foreach(var customer in customers)
{
    Console.WriteLine("\t" + customer);
}
Console.WriteLine();

/*
 * By retrieving a list of the items from the function, you mitigate the issue
 * of returning to the function later, as the data is saved as a list to a variable.
 */

Console.WriteLine("_______________________________________");
Console.WriteLine("ENHANCED-PERFORMANCE");
Console.WriteLine();


var customersList = GetCustomersList();

var countList = customersList.Count();
Console.WriteLine($"\tThere are {countList} customers");

foreach (var customer in customersList)
{
    Console.WriteLine("\t" + customer);
}
Console.WriteLine();


// Functions

IEnumerable<Customer> GetCustomers()
{
    Console.WriteLine("Inside GetCustomers()");
    var lines = File.ReadAllLines("../../../customers.csv"); //For some reason the program starts an extra 2 deep into bin/Debug

    foreach(var line in lines)
    {
        var splitLine = line.Split(",");
        yield return new Customer(splitLine[0], int.Parse(splitLine[1]));
    }
}

IEnumerable<Customer> GetCustomersList()
{
    Console.WriteLine("Inside GetCustomersList()");
    var list = new List<Customer>();
    var lines = File.ReadAllLines("../../../customers.csv"); //For some reason the program starts an extra 2 deep into bin/Debug

    foreach (var line in lines)
    {
        var splitLine = line.Split(",");
        list.Add(new Customer(splitLine[0], int.Parse(splitLine[1])));
    }
    return list;
}




record Customer(string Fullname, int Age);